export class Vertex {
  constructor(graph, type, id, data, options = {}) {
    this.graph = graph;
    this.type = type;
    this.id = id;
    this.data = data;
    this.options = options;
    this.edges = {
      from: [],
      to: [],
    };
    this.installedClickCallback = false;
  }

  reset() {
    this.installedClickCallback = false;
  }

  getEdges(type, away) {
    return this.edges[away ? 'from' : 'to'].filter(
      (edge) => edge.type === type,
    );
  }

  setDisplayLabel(label) {
    this.label = label;
    this.displayVertex();
  }

  displayVertex() {
    if (this.options.hide) {
      return;
    }
    this.graph.flowchart?.log(`${this.id}[${this.label}]`);
    if (this.graph.editable && !this.installedClickCallback) {
      this.graph.flowchart?.log(`click ${this.id} WDGHandler${this.graph.index} "Edit Vertex"`);
      this.installedClickCallback = true;
    }
  }

  static prepareEditorDocument(graph, doc, vertexId) {
    doc.clear();
    const vertex = graph.getVertex(vertexId);
    if (!vertex) {
      throw new Error(`Could not find WDG Vertex ${vertexId}`);
    }
    doc.remark('<h3>Edit Vertex</h3>');
    const form = doc.form().lastElement;
    form
      .textField({
        id: 'id', name: 'id', defaultValue: vertex.id, disabled: true,
      })
      .textField({ id: 'type', name: 'type', defaultValue: vertex.type })
      .textField({ id: 'label', name: 'label', defaultValue: vertex.label })

      .button({
        id: 'save',
        name: 'Save',
        cb: ({ form: { value } }, { initializing }) => {
          if (initializing) return;
          let fullRedraw = false;
          if (value.id && value.id !== vertex.id) {
            fullRedraw = true;
          }
          Object.assign(vertex, value);
          vertex.displayVertex();
          if (fullRedraw) {
            graph.redraw();
          }
        },
      })
        },
      });

    return doc;
  }
}
